#ifndef PATHOLOGY_CPP_IMPLEMENTATION_PATHOLOGY_ANNOTATION_XML_UTILITY_H
#define PATHOLOGY_CPP_IMPLEMENTATION_PATHOLOGY_ANNOTATION_XML_UTILITY_H

#include <map>
#include <string>
#include <pugixml.hpp>
#include <nlohmann/json.hpp>
#include <vector>
#include <chrono>
#include <map>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <fstream>

#ifndef EXPORT_DLL
#define EXPORT_DLL

#define DLL_EXPORT __declspec(dllexport)
#define DLL_IMPORT __declspec(dllimport)

#endif

using namespace std;

DLL_EXPORT pugi::xml_document LoadXml(const string &path);

DLL_EXPORT map<string, string> ParseImageInformation(const pugi::xml_document &root);

DLL_EXPORT map<string, string> ParseUserInformation(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseRectangleAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseLineAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParsePolygonAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseCircleAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseFreehandContourAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseRulerAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseEllipseAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseTextAnnotations(const pugi::xml_document &root);

DLL_EXPORT vector<map<string, string>> ParseLabelList(const pugi::xml_document &root);

DLL_EXPORT map<string, vector<map<string, string>>> GetAllAnnotations(const pugi::xml_document &root);

DLL_EXPORT map<string, string> ParseImageInformation(const string &path);

DLL_EXPORT map<string, string> ParseUserInformation(const string &path);

DLL_EXPORT vector<map<string, string>> ParseRectangleAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseLineAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParsePolygonAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseCircleAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseFreehandContourAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseRulerAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseEllipseAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseTextAnnotations(const string &path);

DLL_EXPORT vector<map<string, string>> ParseLabelList(const string &path);

DLL_EXPORT map<string, vector<map<string, string>>> GetAllAnnotations(const string &path);

DLL_EXPORT void GenerateCOCOJson(const string &path, const string &output_path);

// Methods for Generating COCO Json
DLL_EXPORT string ParseLabelOnAnnotation(const pugi::xml_node &node);

DLL_EXPORT string Join(const vector<string> &vec, const string &delimeter);

DLL_EXPORT vector<double> CalculateBoundingBox(const vector<double> &points);

DLL_EXPORT string GetCurrentYear();

DLL_EXPORT string GetCurrentDateFormatted();

DLL_EXPORT int GetLabelId(const std::string &label, std::map<std::string, int> &labels,
               std::vector<map<string, string>> &categories);

DLL_EXPORT std::vector<double> ParsePoints(const std::string &pointsStr);

DLL_EXPORT nlohmann::json categoriesToJson(const std::vector<std::map<std::string, std::string>> &categories);

DLL_EXPORT void GetRectangles(const std::string &xml_path, nlohmann::json &outputJson,
                   int &annotationCounter, map<string, int> &labels, vector<map<string, string>> &categories);

DLL_EXPORT void GetPolygons(const std::string &xml_path, nlohmann::json &outputJson,
                 int &annotationCounter, map<string, int> &labels, vector<map<string, string>> &categories);

DLL_EXPORT void GetFreehandContours(const std::string &xml_path, nlohmann::json &outputJson,
                         int &annotationCounter, map<string, int> &labels, vector<map<string, string>> &categories);

#endif //PATHOLOGY_CPP_IMPLEMENTATION_PATHOLOGY_ANNOTATION_XML_UTILITY_H
